/*
 * Method to connect WiFi, if no connection,
 * then setup node in AP mode
 */
void initWiFi() {
  Serial.println();
  Serial.println();
  Serial.println("Startup");

  //Close connections
  WiFi.disconnect();
  delay(100);
  WiFi.mode(WIFI_STA);
  Serial.print("Connecting to WiFi ");
  Serial.println(esid);
  Debugln(epass);
  
  //Try to connect to esid
  WiFi.begin((char*)esid.c_str(), (char*)epass.c_str());
  //If there is a connection, configure web server in execution mode
  if ( testWifi() == 20 ) {
    launchWeb(0);
    return;
  }
  //If there isnn't connection, configure node in AP mode
  Serial.println("Opening AP");
  setupAP();
}

/*
 * Method to check if WiFi connection has occurred,
 * and check time out.
 * 
 * If there is connection return values will be 20, otherwise, will be 10
 * 
 */
int testWifi(void) {
  int c = 0;
  Debugln("Wifi test...");
  while ( c < 30 ) {
    if (WiFi.status() == WL_CONNECTED) {
      return (20);
    }
    delay(500);
    Serial.print(".");
    c++;
  }
  Serial.println("WiFi Connect timed out!");
  return (10);
}

/*
 * Method to configure node as an Access Point
 */
void setupAP(void) {

  //Scan available networks
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  int n = WiFi.scanNetworks();
  Serial.println("scan done");
  if (n == 0) {
    Serial.println("no networks found");
    st = "<b>No networks found:</b>";
  } else {
    Serial.print(n);
    Serial.println(" Networks found");
    
    //Save all networks as a html list of networks
    st = "<ul>";
    for (int i = 0; i < n; ++i)
    {
      // Print SSID and RSSI for each network found
      Serial.print(i + 1);
      Serial.print(": ");
      Serial.print(WiFi.SSID(i));
      Serial.print(" (");
      Serial.print(WiFi.RSSI(i));
      Serial.print(")");
      Serial.println((WiFi.encryptionType(i) == ENC_TYPE_NONE) ? " (OPEN)" : "*");

      // Print to web SSID and RSSI for each network found
      st += "<li>";
      st += i + 1;
      st += ": ";
      st += WiFi.SSID(i);
      st += " (";
      st += WiFi.RSSI(i);
      st += ")";
      st += (WiFi.encryptionType(i) == ENC_TYPE_NONE) ? " (OPEN)" : "*";
      st += "</li>";
      delay(10);
    }
    st += "</ul>";
  }
  Serial.println("");
  WiFi.disconnect();
  delay(100);
  
  //Configure & initialize AP mode
  WiFi.mode(WIFI_AP);
  WiFi.softAP(host);
  WiFi.begin(host);
  Serial.print("Access point started with name ");
  Serial.println(host);
  //Set flag that node is working in AP mode
  inApMode = 1;
  //Configure web server in configuration mode
  launchWeb(1);
}

/*
 * Method to start and configure local webserver
 */
void launchWeb(int webtype) {
  Serial.println("");
  Serial.println("WiFi connected");
  //Start the web server or MQTT
  if (webtype == 1) { //in config mode
    Serial.println(WiFi.softAPIP());
    server.on("/", webHandleConfig);
    server.on("/a", webHandleConfigSave);
  } else { // execution mode
    //setup DNS since we are a client in WiFi net
    if (!MDNS.begin(host)) {
      Serial.println("Error setting up MDNS responder!");
    } else {
      Serial.println("mDNS responder started");
      MDNS.addService("http", "tcp", 80);
    }
    Serial.println(WiFi.localIP());
    server.on("/", webHandleRoot);
    server.on("/cleareeprom", webHandleClearRom);

    //Connect MQTT client to broker and configure it
    mqttClient.setServer((char*) mqttServer.c_str(), 1883);
    mqttClient.setCallback(mqtt_arrived);
    mqttClient.setClient(wifiClient);
    //If WiFi is connected and MQTT client is not connected to broker, connect it.
    if (WiFi.status() == WL_CONNECTED) {
      if (!connectMQTT()) {
        delay(2000);
        if (!connectMQTT()) {
          Serial.println("Could not connect MQTT.");
        }
      }
    }
  }
  //Initialize web server
  server.begin();
  Serial.println("Web server started");
}

/*
 * Method that returns HTML code for configuration web
 */
void webHandleConfig() {
  IPAddress ip = WiFi.softAPIP();
  String ipStr = String(ip[0]) + '.' + String(ip[1]) + '.' + String(ip[2]) + '.' + String(ip[3]);
  String s;
  
  s = "CONFIGURACIO DEL NODE " + hostName + " AMB IP: ";
  s += ipStr;
  //List of available WiFi networks
  s += st;
  //Form with parametrer that user has to enter
  s += "<form method='get' action='a'>";
  s += "<label>SSID: </label><input name='ssid' length=32><label> Pass: </label><input name='pass' type='password' length=64></br>";
  s += "</br>";
  //Type of alert field only exists if node is an actuator
  #ifndef ACTUATOR
    s += "<label>Classe d'alerta: </label><select name='alert'><option value='BABY'>Baby</option><option value='DOOR1'>Door 1</option><option value='DOOR2'>Door 2</option><option value='TELEPHONE'>Telephone</option></select></br>";
    s += "</br>";
  #endif
  //Button to send parameters
  s += "<button type='submit'>Guardar...</button></form></p>";
  s += "\r\n\r\n";
  Serial.println("Sending 200");
  //Sends HTML code to web client
  server.send(200, "text/html", s);
}

/*
 * This method saves read parameters that user has been introduced and then 
 * returns HTML code for page that confims to user that parameters has been 
 * saved in the flash memory
 */
void webHandleConfigSave() {
  String s;
  s = "<p>Settings saved to eeprom and reset to boot into new settings</p>\r\n\r\n";
  server.send(200, "text/html", s);
  Serial.println("clearing EEPROM.");
  //Erase actual configuration from flash memory
  clearConfig();
  
  //Reads parameters introduced by user
  String qsid;
  qsid = server.arg("ssid");
  qsid.replace("%2F", "/");
  Serial.println("Got SSID: " + qsid);
  esid = (char*) qsid.c_str();

  String qpass;
  qpass = server.arg("pass");
  qpass.replace("%2F", "/");
  Serial.println("Got pass: " + qpass);
  epass = (char*) qpass.c_str();

  String qalert;
  qalert = server.arg("alert");
  qalert.replace("%2F", "/");
  Serial.println("Got type of alert: " + qalert);
  alert = (char*) qalert.c_str();

  Serial.print("Settings written ");
  //Save configuration to flash memory
  saveConfig() ? Serial.println("sucessfully.") : Serial.println("not succesfully!");
  Serial.println("Restarting!");
  delay(1000);
  ESP.reset();
}

/*
 * Method that returns HTML code for erase configuration from flash memory
 */
void webHandleRoot() {
  String s;
  s = "<p>" +  hostName;
  s += "</p>";
  s += "<a href=\"/cleareeprom\">Clear settings an boot into Config mode</a><br />";
  s += "\r\n\r\n";
  Serial.println("Sending 200");
  server.send(200, "text/html", s);
}

/*
 * This method erase configuration and confirms operation to user
 */
void webHandleClearRom() {
  String s;
  s = "<p>Clearing the config and reset to configure new wifi<p>";
  s += "</html>\r\n\r\n";
  Serial.println("Sending 200");
  server.send(200, "text/html", s);
  Serial.println("clearing config");
  clearConfig();
  delay(10);
  Serial.println("Done, restarting!");
  ESP.reset();
}
